Package: git2rdata
Title: Store and Retrieve Data.frames in a Git Repository
Version: 0.5.2
Authors@R: c(
    person("Thierry", "Onkelinx", , "thierry.onkelinx@inbo.be", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8804-4216", affiliation = "Research Institute for Nature and Forest (INBO)")),
    person("Floris", "Vanderhaeghe", , "floris.vanderhaeghe@inbo.be", role = "ctb",
           comment = c(ORCID = "0000-0002-6378-6229", affiliation = "Research Institute for Nature and Forest (INBO)")),
    person("Peter", "Desmet", , "peter.desmet@inbo.be", role = "ctb",
           comment = c(ORCID = "0000-0002-8442-8025", affiliation = "Research Institute for Nature and Forest (INBO)")),
    person("Els", "Lommelen", , "els.lommelen@inbo.be", role = "ctb",
           comment = c(ORCID = "0000-0002-3481-5684", affiliation = "Research Institute for Nature and Forest (INBO)")),
    person("Research Institute for Nature and Forest (INBO)", , , "info@inbo.be", role = c("cph", "fnd"),
           comment = c(ROR = "00j54wy13"))
  )
Description: The git2rdata package is an R package for writing and reading
    dataframes as plain text files.  A metadata file stores important
    information.  1) Storing metadata allows to maintain the classes of
    variables.  By default, git2rdata optimizes the data for file storage.
    The optimization is most effective on data containing factors.  The
    optimization makes the data less human readable.  The user can turn
    this off when they prefer a human readable format over smaller files.
    Details on the implementation are available in vignette("plain_text",
    package = "git2rdata").  2) Storing metadata also allows smaller row
    based diffs between two consecutive commits.  This is a useful feature
    when storing data as plain text files under version control.  Details
    on this part of the implementation are available in
    vignette("version_control", package = "git2rdata").  Although we
    envisioned git2rdata with a git workflow in mind, you can use it in
    combination with other version control systems like subversion or
    mercurial.  3) git2rdata is a useful tool in a reproducible and
    traceable workflow.  vignette("workflow", package = "git2rdata") gives
    a toy example.  4) vignette("efficiency", package = "git2rdata")
    provides some insight into the efficiency of file storage, git
    repository size and speed for writing and reading.
License: GPL-3
URL: https://ropensci.github.io/git2rdata/,
        https://github.com/ropensci/git2rdata/,
        https://doi.org/10.5281/zenodo.1485309
BugReports: https://github.com/ropensci/git2rdata/issues
Depends: R (>= 4.1.0)
Imports: assertthat, git2r (>= 0.23.0), methods, yaml
Suggests: ggplot2, jsonlite, knitr, microbenchmark, rmarkdown, testthat
VignetteBuilder: knitr
Config/checklist/communities: inbo
Config/checklist/keywords: git; version control; plain text data
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Collate: 'clean_data_path.R' 'data_package.R' 'datahash.R'
        'display_metadata.R' 'git2rdata_package.R' 'is_git2rmeta.R'
        'write_vc.R' 'is_git2rdata.R' 'list_data.R' 'meta.R' 'print.R'
        'prune.R' 'read_vc.R' 'recent_commit.R' 'reexport.R'
        'reinstate.R' 'relabel.R' 'rename_variable.R'
        'update_metadata.R' 'upgrade_data.R' 'utils.R' 'verify_vc.R'
NeedsCompilation: no
Packaged: 2026-04-08 12:52:44 UTC; thierry
Author: Thierry Onkelinx [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8804-4216>, affiliation: Research
    Institute for Nature and Forest (INBO)),
  Floris Vanderhaeghe [ctb] (ORCID:
    <https://orcid.org/0000-0002-6378-6229>, affiliation: Research
    Institute for Nature and Forest (INBO)),
  Peter Desmet [ctb] (ORCID: <https://orcid.org/0000-0002-8442-8025>,
    affiliation: Research Institute for Nature and Forest (INBO)),
  Els Lommelen [ctb] (ORCID: <https://orcid.org/0000-0002-3481-5684>,
    affiliation: Research Institute for Nature and Forest (INBO)),
  Research Institute for Nature and Forest (INBO) [cph, fnd] (ROR:
    <https://ror.org/00j54wy13>)
Maintainer: Thierry Onkelinx <thierry.onkelinx@inbo.be>
Repository: CRAN
Date/Publication: 2026-04-08 19:30:02 UTC
Built: R 4.5.2; ; 2026-04-08 22:25:19 UTC; unix
