# Generated by roxygen2: do not edit by hand

S3method(format,meta_detail)
S3method(format,meta_list)
S3method(is_git2rdata,character)
S3method(is_git2rdata,default)
S3method(is_git2rdata,git_repository)
S3method(is_git2rmeta,character)
S3method(is_git2rmeta,default)
S3method(is_git2rmeta,git_repository)
S3method(list_data,character)
S3method(list_data,default)
S3method(list_data,git_repository)
S3method(meta,Date)
S3method(meta,POSIXct)
S3method(meta,character)
S3method(meta,complex)
S3method(meta,data.frame)
S3method(meta,factor)
S3method(meta,integer)
S3method(meta,logical)
S3method(meta,numeric)
S3method(print,git2rdata)
S3method(print,meta_detail)
S3method(print,meta_list)
S3method(prune_meta,character)
S3method(prune_meta,default)
S3method(prune_meta,git_repository)
S3method(read_vc,character)
S3method(read_vc,default)
S3method(read_vc,git_repository)
S3method(recent_commit,default)
S3method(recent_commit,git_repository)
S3method(relabel,data.frame)
S3method(relabel,default)
S3method(relabel,list)
S3method(rename_variable,character)
S3method(rename_variable,default)
S3method(rename_variable,git_repository)
S3method(rm_data,character)
S3method(rm_data,default)
S3method(rm_data,git_repository)
S3method(summary,git2rdata)
S3method(update_metadata,character)
S3method(update_metadata,default)
S3method(update_metadata,git_repository)
S3method(upgrade_data,character)
S3method(upgrade_data,default)
S3method(upgrade_data,git_repository)
S3method(write_vc,character)
S3method(write_vc,default)
S3method(write_vc,git_repository)
export(commit)
export(data_package)
export(display_metadata)
export(is_git2rdata)
export(is_git2rmeta)
export(list_data)
export(meta)
export(prune_meta)
export(pull)
export(push)
export(read_vc)
export(recent_commit)
export(relabel)
export(rename_variable)
export(repository)
export(rm_data)
export(status)
export(update_metadata)
export(upgrade_data)
export(verify_vc)
export(write_vc)
importFrom(assertthat,"on_failure<-")
importFrom(assertthat,assert_that)
importFrom(assertthat,has_attr)
importFrom(assertthat,has_name)
importFrom(assertthat,is.count)
importFrom(assertthat,is.flag)
importFrom(assertthat,is.string)
importFrom(assertthat,noNA)
importFrom(git2r,add)
importFrom(git2r,blame)
importFrom(git2r,commit)
importFrom(git2r,commits)
importFrom(git2r,hash)
importFrom(git2r,pull)
importFrom(git2r,push)
importFrom(git2r,repository)
importFrom(git2r,status)
importFrom(git2r,workdir)
importFrom(methods,setOldClass)
importFrom(stats,setNames)
importFrom(utils,file_test)
importFrom(utils,flush.console)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(yaml,as.yaml)
importFrom(yaml,read_yaml)
importFrom(yaml,write_yaml)
